/*****************************************************************************
 * vlc_vlvc.h: VLC's core module headers for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2007 the VLVC team
 * $Id: vlc_vlvc.h
 *
 * Authors: Julien Arnaud <arnaud_j@epitech.net>
 *	    Julien Cupillard <cupill_j@epitech.net>
 *          Arnaud Deredec <derede_a@epitech.net>
 *          Florian Gustin <gustin_f@epitech.net>
 *          Guillaume Martin <martin_u@epitech.net>
 *          Florent Nguyen <nguyen_c@epitech.net>
 *
 * This software has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud <http://www.epita.fr/~flav>,
 * Jerome Landrieu and Launiau Severin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/

#ifndef _VLC_VLVC_H
#define _VLC_VLVC_H

#define VLVC_MAX_USERS AOUT_MAX_INPUTS
#define VLVC_MAX_USER_ID_LEN 3

#define VLVC_SERVER_MODE 1
#define VLVC_CLIENT_MODE 2

#define VLVC_SERVER_DEFAULT_INDEX 0

#define VLVC_CONFERENCE_MODE_INVALID -1
#define VLVC_CONFERENCE_MODE_CHAT_ROOM 0
#define VLVC_CONFERENCE_MODE_CONFERENCE (VLVC_CONFERENCE_MODE_CHAT_ROOM + 1)

#define VLVC_MAX_USER_NAME_LENGTH 20
#define VLVC_MAX_USER_DESC_LENGTH 50

#define VLVC_MAX_VIDEO_CODEC_LENGTH 5
#define VLVC_MAX_AUDIO_CODEC_LENGTH 5
#define VLVC_MAX_VIDEO_SCALE_LENGTH 5
#define VLVC_MAX_IP_ADDRESS_LENGTH 16
#define VLVC_MAX_SERVER_INPUT_MRL_LEN (7 + VLVC_MAX_IP_ADDRESS_LENGTH + 1 + VLVC_MAX_USER_ID_LEN + 1)
#define VLVC_MAX_MRL_LEN 1020
#define VLVC_MAX_VLM_COMMAND_LEN 1024

#define VLVC_MAX_NETWORK_PROTOCOL_LEN 10

#define VLVC_MAX_TITLE_OPTION_LEN (VLVC_MAX_USER_NAME_LENGTH + VLVC_SPRINTF_VIDEO_TITLE_OPTION_LEN)

#define VLVC_DEFAULT_VOUT_WIDTH "320"
#define VLVC_DEFAULT_VOUT_HEIGHT "240"

#define VLVC_INVALID_CLIENT_ID -1

#define VLVC_INVALID_FD -1

#define VLVC_IGNORE_VALUE -1

#define VLVC_LOCALHOST "127.0.0.1"

#define VLVC_CREATE_VLM_CONTROL_COMMAND "new vlvcClient%02d broadcast enabled loop"
#define VLVC_SETUP_INPUT_VLM_CONTROL_COMMAND "setup vlvcClient%02d input %s"
#define VLVC_SETUP_OPTION_VLM_CONTROL_COMMAND "setup vlvcClient%02d option %s"
#define VLVC_SETUP_OUTPUT_VLM_CONTROL_COMMAND "setup vlvcClient%02d output %s"
#define VLVC_PLAY_VLM_CONTROL_COMMAND "control vlvcClient%02d play"
#define VLVC_STOP_VLM_CONTROL_COMMAND "control vlvcClient%02d stop"
#define VLVC_DEL_VLM_CONTROL_COMMAND "del vlvcClient%02d"

#define VLVC_SPRINTF_SERVER_INPUT_MRL  "udp://@%s:%d"
#define VLVC_SPRINTF_CLIENT_INPUT_MRL  "udp://@:%d"
#define VLVC_SPRINTF_SELF_SERVER_BEGIN_OUTPUT_MRL "#transcode{vcodec=%s,vb=%d,scale=%s,acodec=%s,ab=%d,channels=2}:duplicate{dst=display,select=video"
#define VLVC_SPRINTF_SELF_SERVER_OUTPUT_MRL VLVC_SPRINTF_SELF_SERVER_BEGIN_OUTPUT_MRL "}"

#define VLVC_SPRINTF_SERVER_BEGIN_OUTPUT_MRL "#duplicate{dst=display"

#define VLVC_SPRINTF_SERVER_STANDALONE_FIRST_NET_DUPLICATE "#duplicate{dst=std{access=%s,mux=ts,dst=%s:%d}"
#define VLVC_SPRINTF_SERVER_NET_DUPLICATE ",dst=std{access=%s,mux=ts,dst=%s:%d}"

#define VLVC_SPRINTF_SELF_CLIENT_OUTPUT_MRL "#transcode{vcodec=%s,vb=%d,scale=%s,acodec=%s,ab=%d,channels=2}:duplicate{dst=std{access=%s,mux=ts,dst=%s:%d},dst=display,select=video}"

#define VLVC_SPRINTF_VIDEO_TITLE_OPTION "video-title=VLVC\\ -\\ %s"
#define VLVC_SPRINTF_VIDEO_TITLE_OPTION_LEN 25

#if defined (WIN32)
# define VLVC_SPRINTF_CACHING_VIDEO_IN_OPTION "dshow-caching=%d"
#else
# define VLVC_SPRINTF_CACHING_VIDEO_IN_OPTION "v4l-caching=%d"
#endif
#define VLVC_SPRINTF_CACHING_UDP_OUT_OPTION   "sout-udp-caching=%d"
#define VLVC_SPRINTF_CACHING_UDP_IN_OPTION    "udp-caching=%d"
#define VLVC_SPRINTF_CACHING_MAX_SIZE 30

#define VLVC_SPRINTF_FORCE_CODEC_MESSAGE "Codecs forced by server:\nVideo: %s @ %dKb/s\nAudio: %s @ %dKb/s"
#define VLVC_SPRINTF_FORCE_CODEC_MESSAGE_MAX_SIZE 255

#define VLVC_OPTION_NO_OVERLAY               "no-overlay"

#define VLVC_SERVER_THREAD      "vlvc server thread"
#define VLVC_CLIENT_THREAD      "vlvc client thread"
#define VLVC_MUTEX              "vlvc_mutex"


#define STRING_TERMINATOR '\0'

#define VLVC_CLIENT_QUIT 1

#define VLVC_NET_MESSAGE_ALL (VLVC_MAX_USERS + 1)
#define VLVC_NET_MESSAGE_TYPE_NORMAL 0
#define VLVC_NET_MESSAGE_TYPE_INFO (VLVC_NET_MESSAGE_TYPE_NORMAL + 1)

#define VLVC_MAX_SEND_TEXT_LEN 1000
#define VLVC_MAX_MESSAGE_LEN   255

/*****************************************************************************
 * Error values
 *****************************************************************************/
#define VLVC_SUCCESS         -0                                   /* No error */
#define VLVC_ENOMEM          -1                          /* Not enough memory */
#define VLVC_ETHREAD         -2                               /* Thread error */
#define VLVC_ETIMEOUT        -3                                    /* Timeout */

#define VLVC_ENOMOD         -10                           /* Module not found */

#define VLVC_ENOOBJ         -20                           /* Object not found */
#define VLVC_EBADOBJ        -21                            /* Bad object type */

#define VLVC_ENOVAR                -30                  /* Variable not found */
#define VLVC_EBADVAR               -31                  /* Bad variable value */

#define VLVC_EEXIT                -255                      /* Program exited */
#define VLVC_EEXITSUCCESS         -999         /* Program exited successfully */
#define VLVC_EGENERIC             -666                       /* Generic error */


#define VLVC_EARGS                 -22 /* invalid arguments given to function */

#define VLVC_VLM_CREATE            -40        /* unable to create vlm control */
#define VLVC_VLM_SETUP             -41         /* unable to setup vlm control */
#define VLVC_VLM_OPTION            -42         /* unable to setup vlm control */

#define VLVC_NO_MORE_USERS_ALLOWED -50                /* no more user allowed */

#define VLVC_NOT_INITIALIZED -32                /* no more user allowed */

#define VLVC_READ_ERROR    0 /* net_Read returns 0 when it's unable to read */
#define VLVC_WRITE_ERROR   0 /* net_Write returns 0 when it's unable to write */


/*****************************************************************************
 * Network commands values
 *****************************************************************************/
#define VLVC_NET_COMMAND_QUIT           1
#define VLVC_NET_COMMAND_HELLO          2
#define VLVC_NET_COMMAND_PING           3
#define VLVC_NET_COMMAND_WELCOME        4
#define VLVC_NET_COMMAND_NOT_WELCOME    5
#define VLVC_NET_COMMAND_ADD_USER       6
#define VLVC_NET_COMMAND_LEAVE_USER     7
#define VLVC_NET_COMMAND_MESSAGE        8
#define VLVC_NET_COMMAND_CODECS         9

typedef struct vlvc_caching_values_t
{
    int i_video_input;
    int i_udp_output;
    int i_udp_input;
} vlvc_caching_values_t;

typedef struct vlvc_user_t
{
    char        psz_name[VLVC_MAX_USER_NAME_LENGTH];
    char        psz_description[VLVC_MAX_USER_DESC_LENGTH];
    char        psz_ip_address[VLVC_MAX_IP_ADDRESS_LENGTH];
    int         i_fd;
    int         b_free;
    int         i_first_client_port;
    
} vlvc_user_t;

struct vlvc_param_t
{
    /* vars */
    int         i_operating_mode;
    int         b_standalone;
    int         i_conference_mode;

    int         b_force_codecs;
    int         b_fix_overlay;

    char        psz_bind_address[VLVC_MAX_IP_ADDRESS_LENGTH];
    int         i_bind_port;
    char        psz_server_address[VLVC_MAX_IP_ADDRESS_LENGTH];
    int         i_server_port;
    int         i_first_client_port;

    char        psz_input_mrl[VLVC_MAX_MRL_LEN];
    char        psz_video_codec[VLVC_MAX_VIDEO_CODEC_LENGTH];
    int         i_video_bitrate;
    char        psz_audio_codec[VLVC_MAX_AUDIO_CODEC_LENGTH];
    int         i_audio_bitrate;
    char        psz_video_scale[VLVC_MAX_VIDEO_SCALE_LENGTH];
    char        psz_username[VLVC_MAX_USER_NAME_LENGTH];
    char        psz_desc[VLVC_MAX_USER_DESC_LENGTH];
    char        psz_network_protocol[VLVC_MAX_NETWORK_PROTOCOL_LEN];
    char        **psz_input_options;
    
    vlvc_caching_values_t caching_values;

};

struct vlvc_t
{
    VLC_COMMON_MEMBERS

    vlc_mutex_t         common_lock;
    vlc_mutex_t         server_lock;
    vlc_mutex_t         client_lock;
    vlc_mutex_t         accept_lock;
    vlc_mutex_t         gui_lock;
    vlc_mutex_t         command_lock;    
    vlc_mutex_t         network_lock;    

    int                 i_nb_users;
    int                 i_self_index;
    int                 i_server_index;
    int                 i_server_base_stream_port;
    vlvc_user_t         p_users[VLVC_MAX_USERS];
    vlvc_param_t        *p_parameters;

    int                 p_pending_fd[VLVC_MAX_USERS];
    int                 i_pending_users;

    int                 b_accept;
    int                 b_started;
    int                 b_initialized;
    int                 b_GUIPresent;

    void                (*NbClientChange) ( int, void * );
    void                (*Stop) ( void * );
    void                (*CoreModuleMessage) ( char *, int, void * );
    void                (*ChatMessage) ( int, const char *, int, void * );

    int                 *p_listen_fd;

    int                 i_client_socket; /* used by both the server and the client */

    int                 i_last_error;

    vlm_t               *p_vlm;
};

typedef struct vlvc_server_user_info_t
{
    char                c_user_id;
    char                psz_name[VLVC_MAX_USER_NAME_LENGTH];
    char                psz_description[VLVC_MAX_USER_DESC_LENGTH];
}vlvc_server_user_info_t;

typedef struct vlvc_client_user_info_t
{
    char                psz_name[VLVC_MAX_USER_NAME_LENGTH];
    char                psz_description[VLVC_MAX_USER_DESC_LENGTH];
    int                 i_first_stream_port;
}vlvc_client_user_info_t;

typedef struct vlvc_welcome_data_t
{
    char                c_user_id;
    char                c_conference_mode;
    char                c_server_id;
    int                 i_first_stream_port;
}vlvc_welcome_data_t;

typedef struct vlvc_message_header_t
{
    char                c_user_src_id;
    char                c_user_dest_id;
    char                c_type;
    int                 i_size;
}vlvc_message_header_t;

typedef struct vlvc_codecs_info_t
{
    char        psz_video_codec[VLVC_MAX_VIDEO_CODEC_LENGTH];
    int         i_video_bitrate;
    char        psz_audio_codec[VLVC_MAX_AUDIO_CODEC_LENGTH];
    int         i_audio_bitrate;
}vlvc_codecs_info_t;

#define vlvc_New( a ) __vlvc_New( VLC_OBJECT( a ) )
VLC_EXPORT( vlvc_t *, __vlvc_New, ( vlc_object_t * ) );
VLC_EXPORT( void, vlvc_Delete, ( vlvc_t * ) );
VLC_EXPORT( int, vlvc_PlayUser, ( vlvc_t *, int ) );
VLC_EXPORT( int, vlvc_StopUser, ( vlvc_t *, int ) );
VLC_EXPORT( int, vlvc_GetUserName, ( vlvc_t *, int , char *) );
VLC_EXPORT( int, vlvc_GetUserDesc, ( vlvc_t *, int , char *) );
VLC_EXPORT( int, vlvc_Init, ( vlvc_t *, vlvc_param_t * ) );
VLC_EXPORT( int, vlvc_Start, ( vlvc_t * ) );
VLC_EXPORT( int, vlvc_Stop, ( vlvc_t * ) );
VLC_EXPORT( int, vlvc_NetSendMessage, ( vlvc_t *, const char *, int ) );
VLC_EXPORT( int, vlvc_SetCallbackChangeNbUsers, (  vlvc_t *, void (*) ( int, void * ) ) );
VLC_EXPORT( int, vlvc_SetCallbackStop, (  vlvc_t *, void (*) ( void * ) ) );
VLC_EXPORT( int, vlvc_SetCallbackCoreModuleMessage, ( vlvc_t *, void (*) ( char *, int, void * ) ) );
VLC_EXPORT( int, vlvc_SetCallbackChatMessage, ( vlvc_t *, void (*) ( int, const char *, int, void *  ) ) );
#endif /*_VLC_VLVC_H*/
