/*****************************************************************************
 * vlc_vlvc_error.h: VLC's core module headers for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2007 the VLVC team
 * $Id: vlc_vlvc_error.h
 *
 * Authors: Julien Arnaud <arnaud_j@epitech.net>
 *	    Julien Cupillard <cupill_j@epitech.net>
 *          Arnaud Deredec <derede_a@epitech.net>
 *          Florian Gustin <gustin_f@epitech.net>
 *          Guillaume Martin <martin_u@epitech.net>
 *          Florent Nguyen <nguyen_c@epitech.net>
 *
 * This software has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud <http://www.epita.fr/~flav>,
 * Jerome Landrieu and Launiau Severin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/

#ifndef _VLC_VLVC_ERROR_H
#define _VLC_VLVC_ERROR_H

/*****************************************************************************
 * Core module critical levels
 *****************************************************************************/
#define VLVC_CRITICAL_LEVEL_MIN          1
#define VLVC_CRITICAL_LEVEL_QUIT_LIMT    5
#define VLVC_CRITICAL_LEVEL_MAX          10
#define VLVC_CRITICAL_TITLE_LOW          "Message"
#define VLVC_CRITICAL_TITLE_HIGH         "Error"

/*****************************************************************************
 * Core module error messages
 *****************************************************************************/

#define VLVC_CORE_MODULE_ERROR_NO_ERROR                      "No error"
#define VLVC_CORE_MODULE_ERROR_NO_ERROR_INDEX                0
#define VLVC_CORE_MODULE_ERROR_BIND                          "Unable to bind specified address"
#define VLVC_CORE_MODULE_ERROR_BIND_INDEX                    (VLVC_CORE_MODULE_ERROR_NO_ERROR_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_SERVER_THREAD                 "Unable to launch server thread"
#define VLVC_CORE_MODULE_ERROR_SERVER_THREAD_INDEX           (VLVC_CORE_MODULE_ERROR_BIND_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_CONNECT                       "Unable to connect to server"
#define VLVC_CORE_MODULE_ERROR_CONNECT_INDEX                 (VLVC_CORE_MODULE_ERROR_SERVER_THREAD_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_CLIENT_THREAD                 "Unable to launch client thread"
#define VLVC_CORE_MODULE_ERROR_CLIENT_THREAD_INDEX           (VLVC_CORE_MODULE_ERROR_CONNECT_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_CONNECTION_LOST               "Connection to the server lost"
#define VLVC_CORE_MODULE_ERROR_CONNECTION_LOST_INDEX         (VLVC_CORE_MODULE_ERROR_CLIENT_THREAD_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_USER_INFO                 "Unable to get user information"
#define VLVC_CORE_MODULE_ERROR_GET_USER_INFO_INDEX           (VLVC_CORE_MODULE_ERROR_CONNECTION_LOST_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE          "User index out of range"
#define VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE_INDEX    (VLVC_CORE_MODULE_ERROR_GET_USER_INFO_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA           "Unable to get conference data"
#define VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA_INDEX     (VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND           "An error occured while waiting for the welcome command"
#define VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND_INDEX     (VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_REFUSED                       "You have been refused by the server"
#define VLVC_CORE_MODULE_ERROR_REFUSED_INDEX                 (VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE            "VLVC can be launched as standalone only in server mode"
#define VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE_INDEX      (VLVC_CORE_MODULE_ERROR_REFUSED_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_STANDALONE_GUI                "Cannot use standalone mode with graphical interface"
#define VLVC_CORE_MODULE_ERROR_STANDALONE_GUI_INDEX          (VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED              "VLVC already launched"
#define VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED_INDEX        (VLVC_CORE_MODULE_ERROR_STANDALONE_GUI_INDEX + 1)

/*****************************************************************************
 * Core module error values
 *****************************************************************************/
char *p_vlvc_error_messages[] = {
    VLVC_CORE_MODULE_ERROR_NO_ERROR,
    VLVC_CORE_MODULE_ERROR_BIND,
    VLVC_CORE_MODULE_ERROR_SERVER_THREAD,
    VLVC_CORE_MODULE_ERROR_CONNECT, 
    VLVC_CORE_MODULE_ERROR_CLIENT_THREAD,
    VLVC_CORE_MODULE_ERROR_CONNECTION_LOST,
    VLVC_CORE_MODULE_ERROR_GET_USER_INFO,
    VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE,
    VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA,
    VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND,
    VLVC_CORE_MODULE_ERROR_REFUSED, 
    VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE, 
    VLVC_CORE_MODULE_ERROR_STANDALONE_GUI, 
    VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED
};

#endif /* _VLC_VLVC_ERROR_H */
