/*****************************************************************************
 * vlvc_frame.hpp: VLC's graphical module headers for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2007 the VLVC team
 * $Id: vlvc_frame.cpp
 *
 * Authors: Julien Arnaud <arnaud_j@epitech.net>
 *	    Julien Cupillard <cupill_j@epitech.net>
 *          Arnaud Deredec <derede_a@epitech.net>
 *          Florian Gustin <gustin_f@epitech.net>
 *          Guillaume Martin <martin_u@epitech.net>
 *          Florent Nguyen <nguyen_c@epitech.net>
 *
 * This module has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud and Jerome Landrieu.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifndef _VLVCFRAME_H_
#define _VLVCFRAME_H_

#include <wx/datetime.h>
#include "wxwidgets.hpp"
#include "vlc_vlvc.h"

#define VLVC_USER_STATUS_ON             1
#define VLVC_USER_STATUS_OFF            0

#define VLVC_NO_USER_NAME		"No name"
#define VLVC_NO_USER_DESC		"No description"

#define VLVC_WINDOW_WIDTH               550
#define VLVC_WINDOW_HEIGHT              300

#define VLVC_CHAT_TEXTBOX_WIDTH         200
#define VLVC_CHAT_TEXTBOX_HEIGHT        100

#define VLVC_SHOW_MESSAGE_EVENT         500
#define VLVC_ADD_USER_EVENT             ( VLVC_SHOW_MESSAGE_EVENT + 1 )
#define VLVC_DEL_USER_EVENT             ( VLVC_ADD_USER_EVENT + 1 )
#define VLVC_CHAT_MESSAGE_EVENT         ( VLVC_DEL_USER_EVENT + 1 )

/* Chat window defines */
#define END_OF_LINE                     "\n"
#define VLVC_TIMESTAMP_FORMAT           "[%H:%M] "
#define VLVC_CHAT_MESSAGE_JOIN          "joined the conference."
#define VLVC_CHAT_MESSAGE_LEAVE         "left the conference."

/* Message colors */
#define VLVC_CHAT_SYSTEM_MSG_COLOR      *wxBLUE
#define VLVC_CHAT_USER_MSG_COLOR        *wxBLACK
#define VLVC_CHAT_DESC_MSG_COLOR        *wxLIGHT_GREY
#define VLVC_CHAT_INFO_MSG_COLOR        wxColour( 0, 128, 0 )
#define VLVC_TEXT_BOX_DEFAULT_TEXT      wxU( _( NULL ) )

#define VLVC_BUTTON_TEXT_PLAY           wxU( _( "Play" ) )
#define VLVC_BUTTON_TEXT_STOP           wxU( _( "Stop" ) )
#define VLVC_BUTTON_TEXT_SEND           wxU( _( "Send" ) )
#define VLVC_FRAME_TEXT_USER            wxU( _( "Users" ) )
#define VLVC_FRAME_TEXT_INFOS           wxU( _( "Infos" ) )
#define VLVC_FRAME_TEXT_ACTION          wxU( _( "Actions" ) )
#define VLVC_FRAME_TEXT_CHAT            wxU( _( "Chat" ) )
#define VLVC_FRAME_TEXT_INFO_USER(user) wxU( "<" ) + (user) + wxU( "> " )

#define VLVC_WINDOW_NAME                "VLVC"

enum
{
    PressEnterEvent,
    SendEvent,
    PlayEvent,
    StopEvent,
    UserList
    /*    ComboSelect*/
};

struct MessageEventData
{
    char psz_message[VLVC_MAX_MESSAGE_LEN];
    int  i_critical;
};

struct ChatMessageEventData
{
    char        psz_message[VLVC_MAX_SEND_TEXT_LEN];
    int         i_user_id;
    int         i_type;
};


namespace wxvlc
{
    struct s_user_info
    {
        int             i_id;
        wxString        name;
        wxString        desc;
    };

    class VlvcFrame: public wxFrame
    {
    public :
        vlvc_t          *p_vlvc;

        VlvcFrame( intf_thread_t *p_intf, wxWindow *p_parent);
        virtual ~VlvcFrame();

    protected:
        intf_thread_t           *p_intf;
        wxWindow                *p_parent;
        char                    p_tab_status[VLVC_MAX_USERS];
        s_user_info             tab_user_info[VLVC_MAX_USERS];
        bool                    b_first_added;

        /* Widgets */
        wxListBox               *p_user_list;
        wxTextCtrl              *p_chat_textbox;
        wxTextCtrl              *p_entry_textbox;

        /* Widget callbacks functions */
        void                    OnButtonSendPress( wxCommandEvent& event );
        void                    OnButtonPlayPress( wxCommandEvent& event );
        void                    OnButtonStopPress( wxCommandEvent& event );
        void                    OnUserListDblClick( wxCommandEvent& event );
        void                    OnCloseWindow(wxCloseEvent& event);

    public :
        void                    OnNbClientChange( int nb );
        void                    OnStopVlvc( );
        void                    OnCoreModuleMessage( char *psz_message, int i_critical );
        void                    OnIncomingChatMessage( int id_user, const char *psz_message, 
                                                       int i_message_type );

    private:
        void                    InitControls();
        wxPanel                 *GetMainPanel( wxFrame *parent );
        wxPanel                 *GetTopPanel( wxPanel *parent );
        wxPanel                 *GetChatPanel( wxPanel *parent );
        wxPanel                 *GetActionPanel( wxPanel *parent );
        wxPanel                 *GetUserPanel( wxPanel *parent );
        wxPanel                 *GetBtnPanel( wxPanel *parent );
        wxPanel                 *GetSendPanel( wxPanel *parent );

        void                    ChatMessage( wxString &message, wxColour &color, bool bShowTimeStamp = true );

        void                    OnMessage( wxCommandEvent& );
        void                    OnAddUser(wxCommandEvent &evt);
        void                    OnDelUser(wxCommandEvent &evt);
        void                    OnChatMessage(wxCommandEvent &evt);
        void                    ShowMessage( char *psz_message, int i_critical );
        void                    AddUser( int );
        void                    DelUser( int );
        
        MessageEventData        messageData;
        ChatMessageEventData    chatMessageData;
        vlc_mutex_t             messageLock;
        int                     b_messageLocked;

        DECLARE_EVENT_TABLE();
    };
}

#endif // ! _VLVCFRAME_H_
